/*
 * HEIF codec.
 * Copyright (c) 2017-2025 Dirk Farin <dirk.farin@gmail.com>
 *
 * This file is part of libheif.
 *
 * libheif is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * libheif is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libheif.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LIBHEIF_HEIF_H
#define LIBHEIF_HEIF_H

#include <libheif/heif_library.h>
#include <libheif/heif_version.h>
#include <libheif/heif_image.h>
#include <libheif/heif_color.h>
#include <libheif/heif_error.h>
#include <libheif/heif_brands.h>

#include <libheif/heif_metadata.h>
#include <libheif/heif_aux_images.h>
#include <libheif/heif_entity_groups.h>
#include <libheif/heif_security.h>
#include <libheif/heif_encoding.h>
#include <libheif/heif_decoding.h>
#include <libheif/heif_context.h>
#include <libheif/heif_image_handle.h>
#include <libheif/heif_tiling.h>
#include <libheif/heif_components.h>
#include <libheif/heif_omaf.h>

#endif
